--> General Library
-- Author: Skere
-- Description: Generally useful Lua library
-- Dependancy: Love2D game engine
--------------------------------------------

local pairs = pairs;

--[[---------------------------------------------------------
   Name:	isValid
   Desc:	Checks if the value is not nil.
------------------------------------------------------------]] 

function isValid( value )

	return ( value ~= nil );

end

--[[---------------------------------------------------------
   Name:	Switch
   Desc:	Elegant elseif statement
------------------------------------------------------------]]

function switch( value, default, ... )

	local cases = { ... };

	for _, v in ipairs( cases ) do

		if ( v[1] == value ) then

			v[2]() return;

		end

	end

	default();

end

--[[---------------------------------------------------------
   Name:	isHovered (love2d only)
   Desc:	Checks if the mouse hovered the given area.
------------------------------------------------------------]]

function isHovered( x, y, w, h )

	local x2 = ( x + w );
	local y2 = ( y + h );
	local mX, mY = love.mouse.getPosition();

	return ( mX >= x and mX <= x2 and mY >= y and mY <= y2 );

end

--[[---------------------------------------------------------
   Name:	printExact
   Desc:	Adds quotation tags to strings when printing
------------------------------------------------------------]]

function printExact( ... )

	local tab = {...};
	local result = {};
	local n = 1;

	for _, v in ipairs( tab ) do

		if ( type( v ) == "string" ) then

			result[n] = "\""..v.."\"";

		else

			result[n] = v;

		end

		n = n + 1;

	end

	print( unpack( result ) );

end

--[[---------------------------------------------------------
   Name:	color
   Desc:	Returns a color table
------------------------------------------------------------]]

function color(r, g, b, a)

	local result;

	if (not g) then -- color(x)
		result = {r, r, r, 255};
	elseif (not b) then -- color(x, x)
		result = {r, r, r, g};
	elseif (not a) then -- color(x, x, x)
		result = {r, g, b, 255};
	else  -- color(x, x, x, x)
		result = {r, g, b, a};
	end

	return result;

end

--[[---------------------------------------------------------
   Name:	required
   Desc:	Returns an error if one of the parameters is nil
------------------------------------------------------------]]

function required(nFields, funcName, ...)

	local parameters = {...};
	local msg = "Function " .. funcName .. "() requires " .. nFields .. " parameters.";

	if (#parameters < nFields) then
		error(msg);
	end

	for _, v in ipairs(parameters) do
		if (not isValid(v)) then
			error(msg);
		end
	end
end